@@ -387,7 +387,7 @@ KDE_NO_EXPORT void PlaylistItem::closed () {
 }
 
 KDE_NO_EXPORT void PlaylistItem::begin () {
-    if (playmode && firstChild ())
+    if (playmode && firstChild () && id_node_ref_url != firstChild ()->id)
         firstChild ()->activate ();
     else
         Mrl::begin ();
