@@ -15,6 +15,7 @@ extern "C" {
 #include <hildon/hildon-seekbar.h>
 #include <hildon/hildon-touch-selector.h>
 #include <hildon/hildon-picker-dialog.h>
+#include <mce/dbus-names.h>
 //#include <gdk/gdk.h>
 //#include <pango/pango-font.h>
 
@@ -389,11 +390,10 @@ void Control::deinit () {
     delete downloader;
 }
 
-static bool needsScreenBlanking (Application *app) {
+static bool needsDeviceMonitoring (Application *app) {
     return app->top_most_window &&
         CONTROL(app)->playlist_data->current &&
-        CONTROL(app)->playlist_data->document->state != Node::state_deferred &&
-        app->fullscreen;
+        CONTROL(app)->playlist_data->document->state != Node::state_deferred;
 }
 
 gboolean cb_blankingTimer (void *p) {
@@ -401,8 +401,18 @@ gboolean cb_blankingTimer (void *p) {
     debugLog() << "cb_blankingTimer" << endl;
 #endif
     Application *app = static_cast <Application *> (p);
-    if (needsScreenBlanking (app)) {
-        osso_display_blanking_pause (app->osso_context);
+    if (needsDeviceMonitoring (app)) {
+        if (app->fullscreen) {
+            osso_display_blanking_pause (app->osso_context);
+        } else {
+            DBusConnection *conn = (DBusConnection *)
+                osso_get_sys_dbus_connection (app->osso_context);
+            DBusMessage *msg = dbus_message_new_method_call (MCE_SERVICE,
+                    MCE_REQUEST_PATH, MCE_REQUEST_IF, "req_keypad_off_pause");
+            dbus_connection_send (conn, msg, NULL);
+            dbus_connection_flush (conn);
+            dbus_message_unref (msg);
+        }
         return 1;
     }
     CONTROL(app)->blanking_timer = 0;
@@ -566,7 +576,7 @@ void Control::playCurrent () {
             setCurrent (playlist_data->document);
         else { // ugly code duplicate w/ back_request
             for (Node *p = playlist_data->current->parentNode(); p; p = p->parentNode())
-                p->setState (Element::state_activated);
+                p->state = Element::state_activated;
         }
         playlist_data->current->activate ();
         updateViewAreaSizes ();
@@ -620,7 +630,7 @@ bool Control::applyBackRequest() {
     } else {
         setCurrent (back_request);
         for (Node *p = back_request->parentNode(); p; p = p->parentNode())
-            p->setState (Element::state_activated);
+            p->state = Element::state_activated;
         back_request = 0L;
         g_timeout_add(0, playCurrentSchedule, static_cast<void*>(this));
     }
@@ -1057,12 +1067,12 @@ void app_init_control (Application *app) {
 }
 
 void app_update_screen_blanking (Application *a) {
-    bool needs_blanking_pause = needsScreenBlanking (a);
-    if (needs_blanking_pause && !CONTROL(a)->blanking_timer) {
-        CONTROL(a)->blanking_timer = g_timeout_add (25000, cb_blankingTimer, a);
-        osso_display_blanking_pause (a->osso_context); // 60s delay
-    } else if (!needs_blanking_pause && CONTROL(a)->blanking_timer) {
-        g_source_remove (CONTROL(a)->blanking_timer);
+    int mon_timer = CONTROL(a)->blanking_timer;
+    bool needs_blanking_pause = cb_blankingTimer (a);
+    if (needs_blanking_pause && !mon_timer) {
+        CONTROL(a)->blanking_timer = g_timeout_add (50000, cb_blankingTimer, a);
+    } else if (!needs_blanking_pause && mon_timer) {
+        g_source_remove (mon_timer);
         CONTROL(a)->blanking_timer = 0;
     }
 }
@@ -1180,10 +1190,14 @@ void cb_playlist_next (GtkButton * button, void *p) {
 
 void cb_playlist_stop (GtkButton * button, void *p) {
     Control *ctr = (Control *)p;
-    if (GTK_WIDGET_VISIBLE (ctr->m_app->tb_stop)) {
+    if (GTK_WIDGET_VISIBLE (ctr->m_app->tb_stop))
         ctr->process ()->setUserStop ();
-        ctr->stop ();
-    } else if (ctr->current_selected) {
+    ctr->stop ();
+}
+
+void cb_playlist_save (GtkButton * button, void *p) {
+    Control *ctr = (Control *)p;
+    if (ctr->current_selected) {
         ctr->request_download = true;
         jump (ctr->current_selected, ctr);
     } else {
@@ -1777,6 +1791,30 @@ void Control::videoDetected ()
     app_update_screen_blanking (m_app);
 }
 
+void Control::insertMrl (Mrl *mrl, const String &uri)
+{
+    if (mrl && !uri.isEmpty ()) {
+        for (Node *n = mrl; n; n = n->parentNode ()) {
+            Mrl *m = n->mrl ();
+            if (m && m->src == uri)
+                return;
+        }
+        for (Node *n = mrl->firstChild (); n; n = n->nextSibling ()) {
+            Mrl *m = n->mrl ();
+            if (m && m->src == uri)
+                return;
+        }
+
+        NodePtr doc = mrl->document ();
+        Node *n = new GenericURL (doc, uri, String ());
+        n->id = id_node_ref_url;
+        n->setAuxiliaryNode (true);
+        mrl->appendChild (n);
+
+        updateTree (m_app, 0);
+    }
+}
+
 static gboolean downloadPageDestroyed (GtkObject *, Application *app)
 {
     debugLog() << "Download page destroyed\n";
