@@ -726,6 +726,8 @@ KMPLAYER_NO_EXPORT void MPlayer::readEvent (UnixProcess *) {
                         m_aspect = (float) String (tok + 16).toDouble ();
                         if (m_aspect > 0.01)
                             updateAspects ();
+                    } else if (!strncmp (tok + 3, "REF", 3)) {
+                        m_notify->insertMrl (mrl(), URL::decode_string(tok + 7));
                     }
                 } else {
                     char *icy = strstr (tok, "ICY Info");
